@@ -1,14 +1,15 @@
 use anyhow::Result;
 
 use crate::codegen::rust::generate_code;
+use clap::StructOpt;
 
 /// Generates rust code
-#[derive(structopt::StructOpt)]
+#[derive(StructOpt)]
 pub struct Command {
     files: Vec<String>,
 
     /// Output file
-    #[structopt(short = "o")]
+    #[structopt(short = 'o')]
     output_filename: String,
 }
 
