@@ -1,7 +1,7 @@
 use std::{borrow::Cow, io::Write, path::Path, process::Command};
 
 use askama::Template;
-use heck::{CamelCase, SnakeCase};
+use heck::{ToSnakeCase, ToUpperCamelCase};
 
 use super::backend::{
     Backend, DeclarationNames, DeclarationTranslationContext, NamespaceNames, RelativeNamespace,
@@ -39,6 +39,7 @@ pub struct TableField {
     pub required: bool,
     pub serialize_default: Option<Cow<'static, str>>,
     pub deserialize_default: Option<Cow<'static, str>>,
+    pub try_from_code: String,
 }
 
 #[derive(Clone, Debug)]
@@ -84,14 +85,10 @@ pub struct UnionVariant {
 }
 
 #[derive(Clone, Debug)]
-pub struct RpcService {
-    // TODO
-}
+pub struct RpcService {}
 
 #[derive(Clone, Debug)]
-pub struct RpcMethod {
-    // TODO
-}
+pub struct RpcMethod {}
 
 const BINDING_KIND_TYPES: &str = "types";
 
@@ -104,7 +101,7 @@ fn reserve_module_name(path: &str, namespace_names: &mut NamespaceNames<'_, '_>)
 }
 
 fn reserve_type_name(path: &str, declaration_names: &mut DeclarationNames<'_, '_>) -> String {
-    let name = path.to_camel_case().into();
+    let name = path.to_upper_camel_case().into();
     declaration_names
         .declaration_names
         .try_reserve_repeat(BINDING_KIND_TYPES, name, '_')
@@ -128,7 +125,7 @@ fn reserve_rust_enum_variant_name(
     binding_kind: &'static str,
     declaration_names: &mut DeclarationNames<'_, '_>,
 ) -> String {
-    let name = path.to_camel_case().into();
+    let name = path.to_upper_camel_case().into();
     declaration_names
         .declaration_names
         .try_reserve_repeat(binding_kind, name, '_')
@@ -280,6 +277,24 @@ impl Backend for RustBackend {
         let primitive_size;
         let mut serialize_default: Option<Cow<'static, str>> = None;
         let mut deserialize_default: Option<Cow<'static, str>> = None;
+        let mut try_from_code = if matches!(field.assign_mode, AssignMode::Optional) {
+            format!(
+                r#"
+                    if let ::core::option::Option::Some({name}) = value.{name}()? {{
+                        ::core::option::Option::Some(::core::convert::TryInto::try_into({name})?)
+                    }} else {{
+                        ::core::option::Option::None
+                    }}
+                "#,
+                name = name
+            )
+        } else {
+            format!(
+                "::core::convert::TryInto::try_into(value.{name}()?)?",
+                name = name
+            )
+        };
+
         match resolved_type {
             ResolvedType::Struct(
                 decl,
@@ -303,10 +318,10 @@ impl Backend for RustBackend {
                     }
                     AssignMode::Optional => {
                         read_type = format!(
-                            "Option<{}<'a>>",
+                            "::core::option::Option<{}<'a>>",
                             format_relative_namespace(&relative_namespace, ref_name)
                         );
-                        owned_type = format!("Option<{}>", vtable_type);
+                        owned_type = format!("::core::option::Option<{}>", vtable_type);
                         create_trait = format!("WriteAsOptional<{}>", vtable_type);
                     }
                     _ => unreachable!(),
@@ -323,23 +338,40 @@ impl Backend for RustBackend {
                 let owned_name =
                     format_relative_namespace(&relative_namespace, owned_name).to_string();
                 primitive_size = 4;
-                vtable_type = format!("planus::Offset<{}>", owned_name);
+                vtable_type = format!("::planus::Offset<{}>", owned_name);
                 match &field.assign_mode {
                     AssignMode::Required => {
                         read_type = format!(
                             "{}<'a>",
                             format_relative_namespace(&relative_namespace, ref_name)
                         );
-                        owned_type = format!("Box<{}>", owned_name);
+                        owned_type = format!("::planus::alloc::boxed::Box<{}>", owned_name);
                         create_trait = format!("WriteAs<{}>", vtable_type);
+                        try_from_code = format!(
+                            "::planus::alloc::boxed::Box::new(::core::convert::TryInto::try_into(value.{name}()?)?)",
+                            name = name
+                        );
                     }
                     AssignMode::Optional => {
                         read_type = format!(
-                            "Option<{}<'a>>",
+                            "::core::option::Option<{}<'a>>",
                             format_relative_namespace(&relative_namespace, ref_name)
                         );
-                        owned_type = format!("Option<Box<{}>>", owned_name);
+                        owned_type = format!(
+                            "::core::option::Option<::planus::alloc::boxed::Box<{}>>",
+                            owned_name
+                        );
                         create_trait = format!("WriteAsOptional<{}>", vtable_type);
+                        try_from_code = format!(
+                            r#"
+                                if let ::core::option::Option::Some({name}) = value.{name}()? {{
+                                    ::core::option::Option::Some(::planus::alloc::boxed::Box::new(::core::convert::TryInto::try_into({name})?))
+                                }} else {{
+                                    ::core::option::Option::None
+                                }}
+                            "#,
+                            name = name
+                        );
                     }
                     AssignMode::HasDefault(..) => unreachable!(),
                 }
@@ -356,7 +388,7 @@ impl Backend for RustBackend {
                 let owned_name =
                     format_relative_namespace(&relative_namespace, owned_name).to_string();
                 primitive_size = 4;
-                vtable_type = format!("planus::Offset<{}>", owned_name);
+                vtable_type = format!("::planus::Offset<{}>", owned_name);
                 match &field.assign_mode {
                     AssignMode::Required => {
                         read_type =
@@ -367,10 +399,10 @@ impl Backend for RustBackend {
                     }
                     AssignMode::Optional => {
                         read_type = format!(
-                            "Option<{}>",
+                            "::core::option::Option<{}>",
                             format_relative_namespace(&relative_namespace, ref_name_with_lifetime)
                         );
-                        owned_type = format!("Option<{}>", owned_name);
+                        owned_type = format!("::core::option::Option<{}>", owned_name);
                         create_trait = format!("WriteAsOptionalUnion<{}>", owned_name);
                     }
                     AssignMode::HasDefault(..) => unreachable!(),
@@ -394,7 +426,7 @@ impl Backend for RustBackend {
                         );
                     }
                     AssignMode::Optional => {
-                        read_type = format!("Option<{}>", vtable_type);
+                        read_type = format!("::core::option::Option<{}>", vtable_type);
                         owned_type = read_type.clone();
                         create_trait = format!("WriteAsOptional<{}>", vtable_type);
                     }
@@ -411,7 +443,7 @@ impl Backend for RustBackend {
                                 .into()
                         }
                         ResolvedType::Table(_, info, relative_namespace) => format!(
-                            "planus::Offset<{}>",
+                            "::planus::Offset<{}>",
                             format_relative_namespace(relative_namespace, &info.owned_name)
                         )
                         .into(),
@@ -425,7 +457,7 @@ impl Backend for RustBackend {
                             format!("[{}]", vector_offset_type(type_)).into()
                         }
                         ResolvedType::Array(_, _) => todo!(),
-                        ResolvedType::String => "planus::Offset<str>".into(),
+                        ResolvedType::String => "::planus::Offset<str>".into(),
                         ResolvedType::Bool => "bool".into(),
                         ResolvedType::Integer(type_) => integer_type(type_).into(),
                         ResolvedType::Float(type_) => float_type(type_).into(),
@@ -455,10 +487,11 @@ impl Backend for RustBackend {
                                 .into()
                         }
                         ResolvedType::Vector(type_) => {
-                            format!("Vec<{}>", vector_owned_type(type_)).into()
+                            format!("::planus::alloc::vec::Vec<{}>", vector_owned_type(type_))
+                                .into()
                         }
                         ResolvedType::Array(_, _) => todo!(),
-                        ResolvedType::String => "String".into(),
+                        ResolvedType::String => "::planus::alloc::string::String".into(),
                         ResolvedType::Bool => "bool".into(),
                         ResolvedType::Integer(type_) => integer_type(type_).into(),
                         ResolvedType::Float(type_) => float_type(type_).into(),
@@ -466,56 +499,93 @@ impl Backend for RustBackend {
                 }
                 fn vector_ref_type<'a>(type_: &ResolvedType<'a, RustBackend>) -> Cow<'a, str> {
                     match type_ {
-                        ResolvedType::Struct(_, info, relative_namespace) => {
-                            format_relative_namespace(relative_namespace, &info.owned_name)
-                                .to_string()
-                                .into()
-                        }
-                        ResolvedType::Table(_, info, relative_namespace) => {
-                            format_relative_namespace(relative_namespace, &info.owned_name)
-                                .to_string()
-                                .into()
-                        }
-                        ResolvedType::Enum(_, info, relative_namespace, _) => {
+                        ResolvedType::Struct(_, info, relative_namespace) => format!(
+                            "{}<'a>",
+                            format_relative_namespace(relative_namespace, &info.ref_name)
+                        )
+                        .into(),
+                        ResolvedType::Table(_, info, relative_namespace) => format!(
+                            "::planus::Result<{}<'a>>",
+                            format_relative_namespace(relative_namespace, &info.ref_name)
+                        )
+                        .into(),
+                        ResolvedType::Enum(_, info, relative_namespace, _) => format!(
+                            "::core::result::Result<{}, ::planus::errors::UnknownEnumTag>",
                             format_relative_namespace(relative_namespace, &info.name)
-                                .to_string()
-                                .into()
-                        }
+                        )
+                        .into(),
                         ResolvedType::Union(_, _, _) => todo!(),
                         ResolvedType::Vector(type_) => {
                             format!("[{}]", vector_offset_type(type_)).into()
                         }
                         ResolvedType::Array(_, _) => todo!(),
-                        ResolvedType::String => "str".into(),
+                        ResolvedType::String => {
+                            "::planus::Result<&'a ::core::primitive::str>".into()
+                        }
                         ResolvedType::Bool => "bool".into(),
                         ResolvedType::Integer(type_) => integer_type(type_).into(),
                         ResolvedType::Float(type_) => float_type(type_).into(),
                     }
                 }
+                fn vector_try_into_func<'a>(type_: &ResolvedType<'a, RustBackend>) -> &'static str {
+                    match type_ {
+                        ResolvedType::Table(..)
+                        | ResolvedType::Enum(..)
+                        | ResolvedType::Union(..)
+                        | ResolvedType::Vector(..)
+                        | ResolvedType::String => "to_vec_result",
+                        ResolvedType::Array(_, _) => todo!(),
+                        _ => "to_vec",
+                    }
+                }
 
                 let offset_name = vector_offset_type(&*type_);
                 let ref_name = vector_ref_type(&*type_);
                 let owned_name = vector_owned_type(&*type_);
                 primitive_size = 4;
-                vtable_type = format!("planus::Offset<[{}]>", offset_name);
+                vtable_type = format!("::planus::Offset<[{}]>", offset_name);
+
+                try_from_code = if matches!(field.assign_mode, AssignMode::Optional) {
+                    format!(
+                        r#"
+                        if let ::core::option::Option::Some({name}) = value.{name}()? {{
+                            ::core::option::Option::Some({name}.{try_into_func}()?)
+                        }} else {{
+                            ::core::option::Option::None
+                        }}
+                        "#,
+                        name = name,
+                        try_into_func = vector_try_into_func(&*type_)
+                    )
+                } else {
+                    format!(
+                        "value.{name}()?.{try_into_func}()?",
+                        name = name,
+                        try_into_func = vector_try_into_func(&*type_)
+                    )
+                };
                 match &field.assign_mode {
                     AssignMode::Required => {
-                        read_type = format!("planus::Vector<'a, {}>", ref_name);
-                        owned_type = format!("Vec<{}>", owned_name);
+                        read_type = format!("::planus::Vector<'a, {}>", ref_name);
+                        owned_type = format!("::planus::alloc::vec::Vec<{}>", owned_name);
                         create_trait = format!("WriteAs<{}>", vtable_type);
                     }
                     AssignMode::Optional => {
-                        read_type = format!("Option<planus::Vector<'a, {}>>", ref_name);
-                        owned_type = format!("Option<Vec<{}>>", owned_name);
+                        read_type =
+                            format!("::core::option::Option<::planus::Vector<'a, {}>>", ref_name);
+                        owned_type = format!(
+                            "::core::option::Option<::planus::alloc::vec::Vec<{}>>",
+                            owned_name
+                        );
                         create_trait = format!("WriteAsOptional<{}>", vtable_type);
                     }
                     AssignMode::HasDefault(Literal::Vector(v)) if v.is_empty() => {
-                        read_type = format!("planus::Vector<'a, {}>", ref_name);
-                        owned_type = format!("Vec<{}>", owned_name);
+                        read_type = format!("::planus::Vector<'a, {}>", ref_name);
+                        owned_type = format!("::planus::alloc::vec::Vec<{}>", owned_name);
                         create_trait = format!("WriteAsDefault<{}, ()>", vtable_type);
 
                         serialize_default = Some("&()".into());
-                        deserialize_default = Some("planus::Vector::EMPTY".into());
+                        deserialize_default = Some("::planus::Vector::EMPTY".into());
                     }
                     AssignMode::HasDefault(..) => unreachable!(),
                 }
@@ -523,22 +593,27 @@ impl Backend for RustBackend {
             ResolvedType::Array(_, _) => todo!(),
             ResolvedType::String => {
                 primitive_size = 4;
-                vtable_type = "planus::Offset<str>".to_string();
+                vtable_type = "::planus::Offset<str>".to_string();
                 match &field.assign_mode {
                     AssignMode::Required => {
-                        read_type = "&'a str".to_string();
-                        owned_type = "String".to_string();
-                        create_trait = "WriteAs<planus::Offset<str>>".to_string();
+                        read_type = "&'a ::core::primitive::str".to_string();
+                        owned_type = "::planus::alloc::string::String".to_string();
+                        create_trait = "WriteAs<::planus::Offset<str>>".to_string();
                     }
                     AssignMode::Optional => {
-                        read_type = "Option<&'a str>".to_string();
-                        owned_type = "Option<String>".to_string();
-                        create_trait = "WriteAsOptional<planus::Offset<str>>".to_string();
+                        read_type =
+                            "::core::option::Option<&'a ::core::primitive::str>".to_string();
+                        owned_type =
+                            "::core::option::Option<::planus::alloc::string::String>".to_string();
+                        create_trait =
+                            "WriteAsOptional<::planus::Offset<::core::primitive::str>>".to_string();
                     }
                     AssignMode::HasDefault(Literal::String(s)) => {
-                        read_type = "&'a str".to_string();
-                        owned_type = "String".to_string();
-                        create_trait = "WriteAsDefault<planus::Offset<str>, str>".to_string();
+                        read_type = "&'a ::core::primitive::str".to_string();
+                        owned_type = "::planus::alloc::string::String".to_string();
+                        create_trait =
+                            "WriteAsDefault<::planus::Offset<::core::primitive::str>, ::core::primitive::str>"
+                                .to_string();
 
                         serialize_default = Some(format!("{:?}", s).into());
                         deserialize_default = Some(format!("{:?}", s).into());
@@ -558,8 +633,8 @@ impl Backend for RustBackend {
                         deserialize_default = Some(format!("{}", lit).into());
                     }
                     AssignMode::Optional => {
-                        read_type = "Option<bool>".to_string();
-                        owned_type = "Option<bool>".to_string();
+                        read_type = "::core::option::Option<bool>".to_string();
+                        owned_type = "::core::option::Option<bool>".to_string();
                         create_trait = "WriteAsOptional<bool>".to_string();
                     }
                     AssignMode::HasDefault(..) => todo!(),
@@ -578,7 +653,7 @@ impl Backend for RustBackend {
                         deserialize_default = Some(format!("{}", lit).into());
                     }
                     AssignMode::Optional => {
-                        read_type = format!("Option<{}>", vtable_type);
+                        read_type = format!("::core::option::Option<{}>", vtable_type);
                         owned_type = read_type.clone();
                         create_trait = format!("WriteAsOptional<{}>", vtable_type);
                     }
@@ -598,7 +673,7 @@ impl Backend for RustBackend {
                         deserialize_default = Some(format!("{}", lit).into());
                     }
                     AssignMode::Optional => {
-                        read_type = format!("Option<{}>", vtable_type);
+                        read_type = format!("::core::option::Option<{}>", vtable_type);
                         owned_type = read_type.clone();
                         create_trait = format!("WriteAsOptional<{}>", float_type(&typ));
                     }
@@ -618,6 +693,7 @@ impl Backend for RustBackend {
             required: matches!(field.assign_mode, AssignMode::Required),
             serialize_default,
             deserialize_default,
+            try_from_code,
         }
     }
 
@@ -650,11 +726,11 @@ impl Backend for RustBackend {
             ResolvedType::Enum(decl, info, relative_namespace, _) => {
                 owned_type = format_relative_namespace(&relative_namespace, &info.name).to_string();
                 getter_return_type = format!(
-                    "std::result::Result<{}, planus::errors::UnknownEnumTag>",
+                    "::core::result::Result<{}, ::planus::errors::UnknownEnumTag>",
                     owned_type
                 );
                 getter_code = format!(
-                    r#"let value: std::result::Result<{}, _> = {}::from_le_bytes(*buffer.as_array()).try_into();
+                    r#"let value: ::core::result::Result<{}, _> = ::core::convert::TryInto::try_into({}::from_le_bytes(*buffer.as_array()));
                     value.map_err(|e| e.with_error_location(
                         {:?},
                         {:?},
@@ -735,7 +811,7 @@ impl Backend for RustBackend {
         match resolved_type {
             ResolvedType::Table(_, info, relative_namespace) => {
                 owned_type = format!(
-                    "Box<{}>",
+                    "::planus::alloc::boxed::Box<{}>",
                     format_relative_namespace(&relative_namespace, &info.owned_name)
                 );
                 ref_type = format!(
