@@ -1,4 +1,4 @@
-use std::collections::HashSet;
+use std::collections::{btree_map, BTreeMap, HashSet};
 
 use codespan::{FileId, Span};
 use codespan_reporting::diagnostic::Label;
@@ -112,7 +112,13 @@ impl<'a> Translator<'a> {
 
         while let Some(last) = namespace_path.pop() {
             match self.namespaces.entry(namespace_path) {
-                Entry::Occupied(_) => break,
+                Entry::Occupied(mut entry) => {
+                    entry
+                        .get_mut()
+                        .child_namespaces
+                        .insert(last, NamespaceIndex(namespace_index));
+                    break;
+                }
                 Entry::Vacant(entry) => {
                     namespace_path = entry.key().clone();
                     let next_namespace_index = entry.index();
@@ -604,10 +610,11 @@ impl<'a> Translator<'a> {
         decl: &ast::Declaration,
     ) -> Declaration {
         let current_file_id = decl.file_id;
+        let definition_span = decl.definition_span;
         let kind =
             match &decl.kind {
                 ast::TypeDeclarationKind::Table(decl) => DeclarationKind::Table(
-                    self.translate_table(current_namespace, current_file_id, decl),
+                    self.translate_table(current_namespace, current_file_id, definition_span, decl),
                 ),
                 ast::TypeDeclarationKind::Struct(decl) => DeclarationKind::Struct(
                     self.translate_struct(current_namespace, current_file_id, decl),
@@ -767,6 +774,7 @@ impl<'a> Translator<'a> {
         field: &ast::StructField,
         next_vtable_index: &mut u32,
         max_vtable_index: &mut u32,
+        has_id_error: &mut bool,
     ) -> Option<TableField> {
         let type_ = self.translate_type(current_namespace, current_file_id, &field.type_)?;
         self.check_valid_table_field_type(current_file_id, &type_);
@@ -790,10 +798,10 @@ impl<'a> Translator<'a> {
         };
         let mut required = false;
         let mut deprecated = false;
-        let vtable_index = *next_vtable_index;
+        let mut vtable_index = *next_vtable_index;
 
         for m in &field.metadata {
-            match m.kind {
+            match &m.kind {
                 MetadataValueKind::Required => {
                     if type_.kind.is_scalar() {
                         self.ctx.emit_error(
@@ -826,7 +834,39 @@ impl<'a> Translator<'a> {
                     }
                 }
                 MetadataValueKind::Deprecated => deprecated = true,
-                // TODO: add support for MetdataValueKind::Id
+                MetadataValueKind::Id(ast::IntegerLiteral {
+                    span,
+                    value,
+                    is_negative,
+                }) => {
+                    if let Some(new_index) = self.translate_integer_generic::<u32>(
+                        current_file_id,
+                        *span,
+                        *is_negative,
+                        value,
+                    ) {
+                        if matches!(type_.kind, TypeKind::Union(_)) {
+                            if new_index > 0 {
+                                // The specification says that the id assignment for unions specifies the second
+                                // of the two IDs, while we use the first in our code
+                                vtable_index = new_index - 1;
+                            } else {
+                                self.ctx.emit_error(
+                                    ErrorKind::MISC_SEMANTIC_ERROR,
+                                    [Label::primary(current_file_id, *span).with_message(
+                                        "This attribute implies the key will have an id of -1",
+                                    )],
+                                    Some("Id assignments for fields of union type specify the id of the value."),
+                                );
+                                vtable_index = 0;
+                            }
+                        } else {
+                            vtable_index = new_index;
+                        }
+                    } else {
+                        *has_id_error = true;
+                    }
+                }
                 _ => {
                     self.emit_metadata_support_error(
                         current_file_id,
@@ -871,6 +911,7 @@ impl<'a> Translator<'a> {
 
         Some(TableField {
             type_,
+            span: field.span,
             assign_mode,
             vtable_index,
             object_value_size: u32::MAX,
@@ -917,6 +958,7 @@ impl<'a> Translator<'a> {
         &self,
         current_namespace: &AbsolutePath,
         current_file_id: FileId,
+        definition_span: Span,
         decl: &ast::Struct,
     ) -> Table {
         let mut next_vtable_index = 0u32;
@@ -931,6 +973,52 @@ impl<'a> Translator<'a> {
             );
         }
 
+        let mut has_id_error = false;
+        let mut first_with_id = None;
+        let mut first_without_id = None;
+        for field in decl.fields.values() {
+            let mut cur_id = None;
+            for m in &field.metadata {
+                if let MetadataValueKind::Id(_) = &m.kind {
+                    if let Some(span) = cur_id {
+                        self.ctx.emit_error(
+                            ErrorKind::MISC_SEMANTIC_ERROR,
+                            [
+                                Label::primary(current_file_id, span)
+                                    .with_message("First attribute was here"),
+                                Label::primary(current_file_id, m.span)
+                                    .with_message("Second attribute was here"),
+                            ],
+                            Some("Field cannot have multiple id attributes"),
+                        );
+                        has_id_error = true;
+                    } else {
+                        cur_id = Some(m.span);
+                    }
+                }
+            }
+
+            if cur_id.is_some() {
+                first_with_id.get_or_insert(field.span);
+            } else {
+                first_without_id.get_or_insert(field.span);
+            }
+        }
+
+        if let (Some(first_with_id), Some(first_without_id)) = (first_with_id, first_without_id) {
+            self.ctx.emit_error(
+                ErrorKind::MISC_SEMANTIC_ERROR,
+                [
+                    Label::primary(current_file_id, first_with_id)
+                        .with_message("First field with an id assignment was here"),
+                    Label::primary(current_file_id, first_without_id)
+                        .with_message("First field without an id assignment was here"),
+                    Label::secondary(current_file_id, definition_span).with_message("Offending table was here")
+                ],
+                Some("Table contains both fields with and without the id assignments, which is disallowed"),
+            );
+        }
+
         let fields = decl
             .fields
             .iter()
@@ -943,10 +1031,70 @@ impl<'a> Translator<'a> {
                         field,
                         &mut next_vtable_index,
                         &mut max_vtable_index,
+                        &mut has_id_error,
                     )?,
                 ))
             })
-            .collect();
+            .collect::<IndexMap<_, _>>();
+
+        let mut seen_vtable_ids: BTreeMap<u32, Span> = BTreeMap::new();
+        let mut insert = |vtable_index, span| match seen_vtable_ids.entry(vtable_index) {
+            btree_map::Entry::Vacant(entry) => {
+                entry.insert(span);
+            }
+            btree_map::Entry::Occupied(entry) => {
+                if !has_id_error {
+                    self.ctx.emit_error(
+                        ErrorKind::MISC_SEMANTIC_ERROR,
+                        [
+                            Label::primary(current_file_id, *entry.get())
+                                .with_message("First id assignment was here"),
+                            Label::primary(current_file_id, span)
+                                .with_message("Second id assignment was here"),
+                        ],
+                        Some(&format!(
+                            "Overlapping id assignments for id {}",
+                            vtable_index
+                        )),
+                    );
+                    has_id_error = true;
+                }
+            }
+        };
+
+        for field in fields.values() {
+            insert(field.vtable_index, field.span);
+            if matches!(field.type_.kind, TypeKind::Union(_)) {
+                insert(field.vtable_index + 1, field.span);
+            }
+        }
+
+        let mut next_expected_id = 0;
+        for &key in seen_vtable_ids.keys() {
+            if key != next_expected_id {
+                let msg = if key == next_expected_id + 1 {
+                    format!(
+                        "Table contains non-consecutive ids. Missing id {}",
+                        next_expected_id
+                    )
+                } else {
+                    format!(
+                        "Table contains non-consecutive ids. Missing ids {}..{}",
+                        next_expected_id,
+                        key - 1
+                    )
+                };
+                if !has_id_error {
+                    self.ctx.emit_error(
+                        ErrorKind::MISC_SEMANTIC_ERROR,
+                        [Label::primary(current_file_id, definition_span)],
+                        Some(&msg),
+                    );
+                }
+            }
+            next_expected_id = key + 1;
+        }
+
         Table {
             fields,
             alignment_order: Vec::new(),
